;;these are global level variables that can be accessed by individuals, groups, and the observer
globals [
emptyCell
totalCultTransEvents
totalGroupExtinctions
totalGroupFissions
timeToCalculate
totalDiversity
totalDiversityList
totalFrequencyList
H_t
H_s
t_f
ShannonsIndex
meanDiffFromAncestral
CVDiffFromAncestral
maxDiffFromAncestral
meanHofGroups
varHofGroups
meanDiffOtherGroups
CVDiffOtherGroups
totalNumberFissions
meanNumberGroupFissions
varNumberGroupFissions
meanSIofGroups
varSIofGroups
totalCopyErrors
totalIntergroupTransEvents
networkDataList
;;rewireProbability  ;; (slider on interface) represents the proportion of patches that will reset, or "rewire," one of their links FROM a von neumann neighbor TO any patch that is not a von neumann neighbor.
]

;;these are the state variables of individuals
turtles-own [
culturalVariant
age
]

;;these are the state variables of groups
patches-own [
linkedPatches
modeCulturalVariant
H
ShannonsIndexGroup
differenceFromAncestral
differenceToAllOtherGroups
withinGroupDiversity
withinGroupDiversityList
withinGroupFrequencyList
withinGroupHtList
fissions
extinctions
]


to setup  ;;this is the first method run when starting a simulation run
ca

;;seed the random number generator
random-seed seed

set timeToCalculate 0
set totalCopyErrors 0
set totalIntergroupTransEvents 0

let j 1     ;;Each patch will get a unique plabel number, 1-100, for display on the interface and for use in the creation of the stochastic network.

;;create the starting population of agents
ask patches
  [

  sprout maxGroupSize [initIndividual]  ;;this tells each patch to create individuals, the number of which is defined by maxGroupSize

  set fissions 0
  set extinctions 0
  set linkedPatches (list)
  set plabel j                ;;This plabel is only used for visualization on the interface.

  if intergroupTransmissionType = "stochastic network"   ;;the preference attachment, or stochastic, network "grows" as each group is added
  [
  establishLink               ;;create the groups' lists of linkedPatches for the stochastic network  (see submodel below)
  ]

  set j j + 1                 ;;increment "j" so it is different for the next patch
  ]

if intergroupTransmissionType = "stochastic network"
 [
 collectNetworkData     ;;collect data on the stochastic network after all groups have been added
 ]


if intergroupTransmissionType = "spatial"   ;;create the groups' lists of linkedPatches for the spatial social network
 [
  ask patches
  [
   ask other patches in-radius intergroupTransmissionRadius   ;;ask each of the other patches within intergroupTransmissionRadius of ego...
   [
   set linkedPatches lput myself linkedPatches                ;;...add ego to your list of linkedPatches.
   ;;show linkedPatches  ;;LSP for testing only
   ]
  ]
 ]


if intergroupTransmissionType = "small world"    ;;create the groups' lists of linkedPatches for the small world network
 [
  ask patches  ;;First, connect each patch to its four von neumann neighbors.
  [
   ask neighbors4
   [
   set linkedPatches lput myself linkedPatches
   ;;show linkedPatches  ;;LSP for testing only
   ]
  ]
 ;;Then use rewireProbability, which represents the proportion of patches that will "rewire" their connection FROM one of their von neumann neighbors TO a patch that is not one of their von neumann neighbors.
 ask n-of round(count patches * rewireProbability) patches
 [
 let x pxcor
 let y pycor
 let oldLink one-of neighbors4                                ;;set oldLink to one of the von neumann neighbors--this is the link that will be replaced by the new one
 let newLink one-of other patches with [distancexy x y  > 1]  ;;this ensures that ego cannot link to itself or to one of its von neumann neighbors

 ;;show linkedPatches  ;;LSP for testing

 set linkedPatches remove oldLink linkedPatches     ;;remove the old link, which is one of the von neumann neighbors

 ask oldLink
 [
 set linkedPatches remove myself linkedPatches     ;;ask the old patch to remove ego from its list of linkedPatches
 ]

 set linkedPatches lput newLink linkedPatches      ;;add the newly linked patch, which is NOT one of the von neumann neighbors

 ask newLink
 [
 set linkedPatches lput myself linkedPatches       ;;ask the new patch to add ego to its list of linkedPatches
 ]
 ;;show linkedPatches  ;;LSP for testing
 ]
];;ends setup of small world network


reset-ticks  ;;set ticks to zero
end



to initIndividual  ;;called by the "setup" method

if equilibrium = TRUE    ;;this setting is used to collect data pertinent to the frequency-based measures of variation: F_ST
[
set culturalVariant random 100000000
]

if equilibrium = FALSE   ;;this setting is used to collect data pertinent to the distance-based measures of variation: d_g and d_amax
[
set culturalVariant 0    ;;every forager starts with the same cultural variant: 0.
]

set age 0
set color blue     ;;all individuals are blue at the start
end




to go  ;;This is the model's "step" function, this is executed during each and every time step.  The order in which other methods are invoked within this method is important.

;;All turtles alive at the beginning of the time step get older; they are now members of the experienced, or "parental," generation.
ask turtles [set age 1]


;;All groups are exposed to local group extinction.
ask patches
[
if random-float 1 < groupExtinctionProb  ;;if a random uniform float is less than group extinction prob
 [
 ask turtles-here [die]   ;;all of the individuals in that group die and are removed from the simulation immediately
 set extinctions extinctions + 1   ;;increment the number of the group's extinctions by one
 set totalGroupExtinctions totalGroupExtinctions + 1   ;;global variable: increment the total number of group extinctions (i.e., all extinctions of all groups)
 ]
]


;;Recolonize empty patches.
ask patches with [count turtles-here = 0]  ;;empty patches initiate the recolonization process
[
 set emptyCell self
 ifelse localReplacement = true   ;;if true, this means that the colonizing group can only come from emptyCell's local Moore neighborhood
    [
      ask one-of neighbors with [count turtles-here > 1] ;;just choose one of the local neighbors at random and ask it...
        [
        colonize   ;;...to colonize emptyCell (see "colonize" method below) ...
        set fissions fissions + 1 ;;increment the number of times this group has fissoned
        set totalGroupFissions totalGroupFissions + 1    ;;global variable, increment the total number of fissioning events.
        ]
    ]
    [;;else, the colonizing group can come from anywhere in the world
       ask one-of patches with [count turtles-here > 1]   ;;just choose one of the populated groups at random from the whole world and ask it...
        [
        colonize  ;;...to colonize emptyCell (see "colonize" method below) ...
        set fissions fissions + 1 ;;increment the number of times this group has fissoned
        set totalGroupFissions totalGroupFissions + 1  ;;global variable, increment the total number of fissioning events.
        ]
    ]
]



;;Produce new generation of turtles in each patch. The new turtles will eventually learn their culturalVariant from one of the "experienced" turtles in their patch (see cultTransmission).
ask patches
[
sprout maxGroupSize
 [
 set age 0              ;;this signifies that the agent is member of the "naive" generation during this time step
 set culturalVariant 0  ;;this 0 will ultimately be replaced by a value in the method "cultTransmission" (see below)
 set color blue
 ]
]


;;The "naive" generation of turtles learns from the "experienced" generation via unbiased cultural transmission within groups.
ask turtles with [age = 0]
[
cultTransmission    ;;see this submodel below
]

;;after cultural transmission, kill off all turtles with age = 1 ("parental" generation)
ask turtles with [age = 1]
[
die
]

;;Intergroup cultural transmission.
if Nm > 0  ;;when Nm = 0, there is no intergroup cultural transmission
[
    ifelse Nm <= maxGroupSize
    [
     ask patches [intergroupTransmission]     ;;see this submodel below
    ]
    [
     error "Nm is incorrectly set to a value greater than maxGroupSize. Decrease Nm and try again."
    ]
]


;;Time passes, this increments "ticks" by one
tick


;;Calculate measures so that the graphs and plots can be updated in the interface and data can be collected at intervals throughout the simulation run if needed.
if ticks = timeToCalculate or ticks = 1   ;; if it is time to "refresh" diversity measures
 [
 ;;show ticks  ;;LSP testing
 calculateTotalDiversity  ;;calculate lists of unique variants and their relative frequencies, THIS MUST BE DONE BEFORE THE DIVERSITY MEASURES ARE CALCULATED BELOW, THEY RELY UPON IT

 ;;calculate global measure of average cultural diversity (multiple variants at a single locus), Ht (Nei and Kumar 2000: equ. 12.43)
 ask patches
  [
  calculateWithinGroupPortionOfHt  ;;see method below
  ]
 calculateHt                       ;;see method below, NOTE: this *must* come after calculateWithinGroupPortionOfHt
 calculateShannonIndex             ;;calculate another global measure of average cultural diversity  (see method below)
 calculateNumberUniqueVariantsWithinGroups
 calculateShannonIndexGroups
 calculateHs
 calculateH



;;update the modal lists of the patches after individuals have completed cultTransmission
ask patches  ;;ask all patches to reset the modes of each of their traits to the mode of the individuals they contain (ties for modes broken randomly)
[
set modeCulturalVariant one-of modes [culturalVariant] of turtles-here
]


ask patches
[
if any? turtles-here
 [
 calculateDifferenceFromAncestral   ;;for all patches that contain individuals, calculate their cultural difference from the ancestral state
 ]
]

ask patches
[
if any? turtles-here
 [
 calculateDifferenceToAllOtherGroups  ;;for all patches that contain individuals, calculate their cultural difference from all other groups
 ]
]


;;recolor patches, such that the group that is most different from ancestral is white and all others a darker shade of red
if equilibrium = FALSE
[
 let maximumDifference max [differenceFromAncestral] of patches
 ask patches
 [
 set pcolor scale-color red differenceFromAncestral 0 (maximumDifference + 0.000001)  ;;"red" can be changed to any color you prefer
 ]
]


 if collectData = true  ;;if one would like to collect data
 [
 ;;set remaining variables here
 set meanDiffFromAncestral mean([differenceFromAncestral] of patches)       ;;this is interpretable only when equilibrium = FALSE
 set maxDiffFromAncestral max([differenceFromAncestral] of patches)         ;;this is interpretable only when equilibrium = FALSE
 ifelse meanDiffFromAncestral > 0
 [
 set CVDiffFromAncestral (standard-deviation([differenceFromAncestral] of patches) / mean([differenceFromAncestral] of patches))     ;;this is interpretable only when equilibrium = FALSE
 ]
 [
 set CVDiffFromAncestral 0                         ;;this is interpretable only when equilibrium = FALSE
 ]

 set meanNumberGroupFissions mean([fissions] of patches)
 set varNumberGroupFissions variance([fissions] of patches)
 set meanHofGroups mean([H] of patches)
 set meanSIofGroups mean([ShannonsIndexGroup] of patches)
 set varSIofGroups variance([ShannonsIndexGroup] of patches)
 set varHofGroups variance([H] of patches)
 set meanDiffOtherGroups mean([differenceToAllOtherGroups] of patches)        ;;this is interpretable only when equilibrium = FALSE
 ifelse meanDiffOtherGroups > 0
 [
 set CVDiffOtherGroups (standard-deviation([differenceToAllOtherGroups] of patches) / mean([differenceToAllOtherGroups] of patches))    ;;this is interpretable only when equilibrium = FALSE
 ]
 [
 set CVDiffOtherGroups 0                           ;;this is interpretable only when equilibrium = FALSE
 ]

 ;;printOutput   ;;uncommenting this line will allow for the data to be written to the textfile throughout the simulation run.  Note that this might cause some confusion in the textfile when running BehaviorSpace experiments on multiple processors at once.
 ]

 set timeToCalculate timeToCalculate + refreshRate   ;;reset timeToCalculate so that data are collected again in due time according to the global variable, refreshRate.
]


;;Update the plots on interface with "fresh" data.

if equilibrium = FALSE
[
set-current-plot "Difference from Ancestral State (Histogram)"     ;;this is interpretable only when equilibrium = FALSE
clear-plot
auto-plot-on
histogram [differenceFromAncestral] of patches

set-current-plot "Difference from Ancestral State"                 ;;this is interpretable only when equilibrium = FALSE
auto-plot-on
set-current-plot-pen "mean"
plot mean [differenceFromAncestral] of patches
set-current-plot-pen "sd"
plot standard-deviation [differenceFromAncestral] of patches
set-current-plot-pen "var"
plot variance [differenceFromAncestral] of patches
ifelse mean [differenceFromAncestral] of patches > 0
[
set-current-plot-pen "CV"
plot (standard-deviation [differenceFromAncestral] of patches) / (mean [differenceFromAncestral] of patches)
]
[
set-current-plot-pen "CV"
plot 0
]


set-current-plot "Difference from All Other Groups"                ;;this is interpretable only when equilibrium = FALSE
auto-plot-on
set-current-plot-pen "mean"
plot mean [differenceToAllOtherGroups] of patches
set-current-plot-pen "sd"
plot standard-deviation [differenceToAllOtherGroups] of patches
set-current-plot-pen "var"
plot variance [differenceToAllOtherGroups] of patches
ifelse mean [differenceToAllOtherGroups] of patches > 0
[
set-current-plot-pen "CV"
plot (standard-deviation [differenceToAllOtherGroups] of patches) / (mean [differenceToAllOtherGroups] of patches)
]
[
set-current-plot-pen "CV"
plot 0
]


set-current-plot "Difference from All Other Groups (Histogram)"     ;;this is interpretable only when equilibrium = FALSE
clear-plot
auto-plot-on
histogram [differenceToAllOtherGroups] of patches
]



set-current-plot "Group Cultural Diversity"
auto-plot-on
set-current-plot-pen "mean"
plot mean [H] of patches
set-current-plot-pen "sd"
plot standard-deviation [H] of patches
set-current-plot-pen "var"
plot variance [H] of patches
set-current-plot-pen "CV"
ifelse mean [H] of patches > 0
[
plot (standard-deviation [H] of patches) / (mean [H] of patches)
]
[
plot (0)
]


set-current-plot "Population Cultural Diversity"
auto-plot-on
set-current-plot-pen "Total"
plot H_t
set-current-plot-pen "Within"
plot H_s
set-current-plot-pen "Between"
plot (H_t - H_s)


;;Stop condition
if ticks = 50000        ;;This is the stop condition for all simulation runs.  If you want the sims to run longer or shorter, change this value.  You'll also have to change the stop condition in the BehaviorSpace experiments if you use them after changing this value.
[
 if collectData = TRUE
  [
  printOutput            ;;cultural variation data are written to the textfile at the very end of the simulation run
  collectSpatialData     ;;spatially explicit data are collected in their own file at the very end of the simulation run
  ]
 stop
]
end   ;;this ends the "go" method






to establishLink  ;;patches method for creating a stochastic scale-free (i.e., preference attachment) network as described in Bentley and Shennan (2003)

if any? other patches with [count turtles-here > 0]  ;;if there is at least 1 patch with turtles, then go on with the method.
[
  ifelse count other patches with [count turtles-here > 0] = 1  ;;if there is only one other patch with a group in it...
  [
  let linkedPatch one-of other patches with [count turtles-here > 0]

  set linkedPatches lput linkedPatch linkedPatches     ;;then set my list of linkedPatches to include that patch...

  ask linkedPatch
    [
    set linkedPatches lput myself linkedPatches    ;;and ask that patch to set its list of linkedPatches to include myself.
    ]
  ;;ask patches with [length(linkedPatches) > 0] [set pcolor yellow]  ;;LSP for testing
  ]
  [  ;;else there is more than 1 other patch with turtles in it

  let tempList (list)  ;;build the temporary list, called tempList
  let possibleLinks other patches with [count turtles-here > 0]  ;;build a local agentset, called possibleLinks, that includes all other patches that have turtles

  ask possibleLinks  ;;for each patch in possibleLinks (that patch becomes ego)
    [
    let i length(linkedPatches)  ;;set i to the number of patches to which ego is linked
    while [i > 0]
    [
    set tempList lput plabel tempList    ;;add ego's plabel to tempList once for each patch to which it is linked.  If ego has plabel '3' and is linked to 5 other patches, then the result of this while loop is adding the number '3' to tempList 5 times.
    set i (i - 1)
    ]
    ;;show tempList  ;;LSP for testing
    ]

  ;;show tempList  ;;LSP for testing

  ;;show one-of tempList  ;;LSP for testing

  let p one-of tempList   ;;local variable, p, is set to the plabel of a randomly chosen value from tempList

  let linkedPatch one-of patches with [plabel = p]    ;;local variable is set to the patch identified by the plabel chosen from tempList

  set linkedPatches lput linkedPatch linkedPatches    ;;then set my list of linkedPatches to include that patch...

  ask linkedPatch
    [
    set linkedPatches lput myself linkedPatches      ;;and ask that patch to set its list of linkedPatches to include myself.
    ]
  ]
]
end




to intergroupTransmission  ;;patches method

ifelse intergroupTransmissionType = "spatial"   ;;this is the local intergroup transmission option, the "teacher" comes from a patch within intergroupTransmissionRadius of ego (ego is a patch because this is a patches method)
[
ask n-of Nm turtles-here   ;;Nm turtles will replace the culturalVariant value they got through unbiased cultural transmission with the value acquired here via horizontal intergroup cultural transmission
  [
  ;;write "I am about to learn from outside my group. My culturalVariant is: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing

  set culturalVariant [culturalVariant] of one-of turtles-on one-of linkedPatches    ;;the teacher is a member of any of the linkedPatches
  set totalIntergroupTransEvents totalIntergroupTransEvents + 1                      ;;increment the total number of intergroup transmission events

  ;;write "I have learned from outside my group. My culturalVariant is now: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing
  ]
]
[
ifelse intergroupTransmissionType = "stochastic network" or intergroupTransmissionType = "small world"  ;;both of these options make use of the linkedPatches agentset
[
ask n-of Nm turtles-here  ;;Nm turtles will replace the culturalVariant value they got through unbiased cultural transmission with the value acquired here via horizontal intergroup cultural transmission
  [
  ;;write "I am about to learn from someone outside my group. My culturalVariant is: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing

  set culturalVariant [culturalVariant] of one-of turtles-on one-of linkedPatches  ;;the teacher is a member of any of the linkedPatches
  set totalIntergroupTransEvents totalIntergroupTransEvents + 1                    ;;increment the total number of intergroup transmission events

  ;;write "I have learned from someone outside my group. My culturalVariant is now: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing
  ]
]
[  ;;else intergroupTransmissionType is set to global.  The "teacher" for each turtle comes from any patch other than ego, regardless of how far away it is from ego
ask n-of Nm turtles-here    ;;Nm turtles will replace the culturalVariant value they got through unbiased cultural transmission with the value acquired here via horizontal intergroup cultural transmission
  [
  ;;write "I am about to learn from someone outside my group. My culturalVariant is: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing

  set culturalVariant [culturalVariant] of one-of turtles-on one-of other patches  ;;the teacher is a member of any patch other than ego's patch, note that linkedPatches isn't used here because there is no need to create a subset of linked groups
  set totalIntergroupTransEvents totalIntergroupTransEvents + 1                    ;;increment the total number of intergroup transmission events

  ;;write "I have learned from someone outside my group. My culturalVariant is now: "  ;;LSP for testing
  ;;print culturalVariant   ;;LSP for testing
  ]
]
]
end




to cultTransmission    ;;turtle method, this operationalizes unbiased cultural transmission from the "experienced" to the "naive" generation, and it includes single stepwise copying errors

let teacher one-of turtles-here with [age = 1]  ;; else it adopts the traits displayed by a randomly chosen individual from the previous generation within the group
;;show teacher  ;for testing
set culturalVariant [culturalVariant] of teacher


;;Next allow for some copy error to modify the naive agent's cultural variant in a single stepwise fashion.

if innovationType = "bidirectional"  ;;an innovation is equally likely to increase or decrease the variant by one
[
if random-float 1 < errorProb   ;;if a random uniform float is less than or equal to errorProb, then there will be an innovation
 [
  ifelse random-float 1 < 0.5   ;;add or subtract 1 with equal probability
    [set culturalVariant culturalVariant + 1]
    [set culturalVariant culturalVariant - 1]
  set totalCopyErrors totalCopyErrors + 1  ;;increment the total number of copy errors in the simulation run
 ]
]


;; forward only single stepwise copy error; this one guards against "back-innovations" that can  occur under the "both-ways" version
if innovationType = "forward-only"
[
if random-float 1 < errorProb
 [
  set culturalVariant culturalVariant + 1  ;;always add 1 to the current value of culturalVariant
  set totalCopyErrors totalCopyErrors + 1  ;;increment the total number of copy errors in the simulation run
 ]

]

set totalCultTransEvents totalCultTransEvents + 1  ;;increment the total number of cultural transmission events in the simulation run

set color culturalVariant    ;;this is just for visualization on the interface
end




to colonize    ;;patch method
ask n-of round(count turtles-here / 2) turtles-here [move-to emptyCell]    ;;send half of the current members (chosen randomly) to the empty cell
end



to calculateTotalDiversity  ;;observer method
set totalDiversityList (list)
;;show totalDiversityList     ;;LSP for testing only
set totalFrequencyList (list)

ask turtles
 [
 let myVariant culturalVariant
 ifelse member? myVariant totalDiversityList = false
  [
  set totalDiversityList lput myVariant totalDiversityList
  ;;show totalDiversityList  ;;LSP for testing only
  set totalFrequencyList lput 1 totalFrequencyList
  ]
  [
  let oldFrequency item (position myVariant totalDiversityList) totalFrequencyList
  ;;show oldFrequency        ;;LSP for testing only
  ;;show totalFrequencyList  ;;LSP for testing only
  set totalFrequencyList replace-item (position myVariant totalDiversityList) totalFrequencyList (oldFrequency + 1)
  ;;show totalFrequencyList  ;;LSP for testing only
  ]
 ]

set totalDiversity length totalDiversityList     ;;this is richness
;;show length totalDiversityList  ;;LSP for testing only
end


to calculateNumberUniqueVariantsWithinGroups      ;;observer method.  calculate Number Of Unique variants Within Groups
 ask patches
 [
  let withinGroupDiversityListTemp (list)
  ;;show withinGroupDiversityListTemp   ;;LSP for testing only
  let withinGroupFrequencyListTemp (list)

  ask turtles-here
  [
  let myVariant culturalVariant
  ifelse member? myVariant withinGroupDiversityListTemp = false
   [
   set withinGroupDiversityListTemp lput myVariant withinGroupDiversityListTemp
   ;;show withinGroupDiversityListTemp  ;;LSP for testing only
   set withinGroupFrequencyListTemp lput 1 withinGroupFrequencyListTemp
   ;;show withinGroupFrequencyListTemp  ;;LSP for testing only
   ]
   [
   let oldFrequency item (position myVariant withinGroupDiversityListTemp) withinGroupFrequencyListTemp
   ;;show oldFrequency                  ;;LSP for testing
   ;;show withinGroupFrequencyListTemp  ;;LSP for testing only
   set withinGroupFrequencyListTemp replace-item (position myVariant withinGroupDiversityListTemp) withinGroupFrequencyListTemp (oldFrequency + 1)
   ;;show withinGroupFrequencyListTemp  ;;for testing only
   ]
  ]
 set withinGroupDiversityList withinGroupDiversityListTemp
 ;;show length withinGroupDiversityList  ;;LSP for testing only
 set withinGroupDiversity length withinGroupDiversityList
 set withinGroupFrequencyList withinGroupFrequencyListTemp
 ]
end




to calculateHs   ;;observer method
 ;;calculate Hs (Nei and Kumar 2000: equ. 12.42)
 let weightedSumOfSquares 0
 ask patches
  [
  let sumOfSquares 0
  foreach withinGroupFrequencyList
   [ ?1 ->
   set sumOfSquares sumOfSquares + ((?1 / count turtles-here) ^ 2)
   ]
  set weightedSumOfSquares weightedSumOfSquares + ((count turtles-here / count turtles)  * sumOfSquares)
  ]

 set H_s (1 - weightedSumOfSquares)
 ;;write "Hs:"   ;;LSP for testing
 ;;print H_s     ;;LSP for testing
end



to calculateH    ;;observer method
 ;;calculate H for each group (Nei and Kumar 2000, Eq. 12.41).
 ask patches
 [
 let sumOfSquares 0

 foreach withinGroupFrequencyList
  [ ?1 ->
  set sumOfSquares sumOfSquares + ((?1 / count turtles-here) ^ 2)
  ]
 set H (1 - sumOfSquares)
 ]
end


to calculateShannonIndex    ;;observer method
let summation 0

foreach totalFrequencyList
 [ ?1 ->
 set summation summation + ((?1 / count turtles) * (ln (?1 / count turtles)))
 ]

set ShannonsIndex (-1 * summation)
;;write "Shannon Index:" ;;LSP for testing
;;print ShannonsIndex    ;;LSP for testing
end


to calculateShannonIndexGroups    ;;observer method
ask patches
[
let summation 0

foreach withinGroupFrequencyList
 [ ?1 ->
 set summation summation + ((?1 / count turtles-here) * (ln (?1 / count turtles-here)))
 ]

set ShannonsIndexGroup (-1 * summation)
;;write "Shannons Index Group:"   ;;LSP for testing
;;print ShannonsIndexGroup        ;;LSP for testing
]
end


to calculateWithinGroupPortionOfHt  ;;patches method
set withinGroupHtList n-values (length totalDiversityList) [0]
;;show withinGroupHtList  ;;LSP for testing only

let numberOfTurtlesInTheGroup count turtles-here

ask turtles-here
 [
 let myVariant culturalVariant

 let oldFrequency item (position myVariant totalDiversityList) [withinGroupHtList] of myself
 ;;show oldFrequency  ;;LSP for testing only

 let newFrequency (oldFrequency + (1 / numberOfTurtlesInTheGroup))
 ;;show newFrequency  ;;LSP for testing only

  ask patch-here
  [
  set withinGroupHtList replace-item (position myVariant totalDiversityList) withinGroupHtList newFrequency
  ]
 ;;show withinGroupHtList  ;;LSP for testing only
 ]
;;show withinGroupHtList  ;;LSP for testing only
end


to calculateHt    ;;observer method
let xbari 0
let sumOfXbariSquared 0
let q 0

while [q < length totalDiversityList]
 [
 set xbari mean [item q withinGroupHtList] of patches
 ;;show xbari  ;;LSP for testing only

 set sumOfXbariSquared sumOfXbariSquared + (xbari ^ 2)
 ;;show sumOfXbariSquared  ;;LSP for testing only

 set q q + 1
 ]

set H_t (1 - sumOfXbariSquared)
;;write "Ht:" ;;LSP for testing
;;print H_t  ;;LSP for testing

set t_f ((1 / sumOfXbariSquared) - 1)
;;write "tf:"  ;;LSP for testing
;;print t_f   ;;LSP for testing
end




to calculateDifferenceFromAncestral  ;;patch method
set differenceFromAncestral (abs(modeCulturalVariant))     ;;This measures how different the mode of the *group* is from the ancestral state of 0.  This can only be interpreted when equilibrium=FALSE.
end



to calculateDifferenceToAllOtherGroups  ;;patch method
  let sumDiff 0
  let currentPatch self

    foreach ([self] of other patches)  ;;compare currentPatch to all other patches
    [ ?1 ->
    let targetPatch ?1  ;;look at each patch in turn
    if count [turtles-here] of targetPatch > 0 and targetPatch != currentPatch     ;;targetPatch must have members and it cannot be the currentPatch
     [
     ask currentPatch
       [
       set sumDiff (sumDiff + (abs (modeCulturalVariant - [modeCulturalVariant] of targetPatch)))  ;;sum the absolute value of the difference between the currentPatch's modal variant and the targetPatch's modal variant
       ;;show sumDiff  ;;LSP for testing
       ]
     ]
   ]  ;;end foreach loop

  set differenceToAllOtherGroups (sumDiff / count other patches with [count turtles-here > 0])   ;;sumDiff divided by the number of other groups used in the pairwise comparisons with currentPatch
  ;;show differenceToAllOtherGroups    ;;LSP for testing
end




to printOutput   ;;observer method, called from within "go"
 ;;write data to file
 file-open "diversityData.txt"   ;;open file and write new header info to it
 file-print " "
 file-write "seed worldSizeX worldSizeY maxGroupSize equilibrium groupExtinctionProb localReplacement innovationType errorProb dataCollectionInterval Nm intergroupTransmissionRadius intergroupTransmissionType rewireProbability" file-print " "
 file-write seed file-write (max-pxcor + 1) file-write (max-pycor + 1) file-write maxGroupSize file-write equilibrium file-write groupExtinctionProb file-write localReplacement file-write innovationType file-write errorProb file-write refreshRate file-write Nm file-write intergroupTransmissionRadius file-write intergroupTransmissionType file-write rewireProbability
 file-print " "
 file-write "Time PopSize Richness SI Ht Hs tF meanDA CVDA maxDA CTE Extinctions Fissions meanFissions varFissions meanH varH meanDO CVDO meanSIGroup varSIGroup" file-print " "
 file-write ticks
 file-write count turtles
 file-write totalDiversity
 file-write ShannonsIndex
 file-write H_t
 file-write H_s
 file-write t_f
 file-write meanDiffFromAncestral
 file-write CVDiffFromAncestral
 file-write maxDiffFromAncestral
 file-write totalCultTransEvents
 file-write totalGroupExtinctions
 file-write totalGroupFissions
 file-write meanNumberGroupFissions
 file-write varNumberGroupFissions
 file-write meanHofGroups
 file-write varHofGroups
 file-write meanDiffOtherGroups
 file-write CVDiffOtherGroups
 file-write meanSIofGroups
 file-write varSIofGroups
 file-print " "
 file-close
end



to collectNetworkData    ;;observer method
let i max [length linkedPatches] of patches
set networkDataList (list)
let networkDataList2 (list)
;;show i  ;;LSP for testing

while [i > 0]
[
set networkDataList fput count patches with [(length linkedPatches) = i] networkDataList
set networkDataList2 fput i networkDataList2
set i (i - 1)
]

;;show networkDataList  ;;LSP for testing
;;show networkDataList2  ;;LSP for testing

if collectData = true
[
 file-open "networkData.txt"   ;;open file and write new header info to it
 file-print " "
 file-write "seed worldSizeX worldSizeY maxGroupSize equilibrium groupExtinctionProb localReplacement innovationType errorProb dataCollectionInterval Nm intergroupTransmissionRadius intergroupTransmissionType" file-print " "
 file-write seed file-write (max-pxcor + 1) file-write (max-pycor + 1) file-write maxGroupSize file-write equilibrium file-write groupExtinctionProb file-write localReplacement file-write innovationType file-write errorProb file-write refreshRate file-write Nm file-write intergroupTransmissionRadius file-write intergroupTransmissionType
 file-print " "
 file-write networkDataList
 file-print " "
 file-write networkDataList2
 file-print " "
 file-close
]
end



to collectSpatialData    ;;observer method
file-open "spatialData.txt"   ;;open file and write new header info to it
file-print " "
file-write "seed worldSizeX worldSizeY maxGroupSize equilibrium groupExtinctionProb localReplacement innovationType errorProb dataCollectionInterval Nm intergroupTransmissionRadius intergroupTransmissionType rewireProbability" file-print " "
file-write seed file-write (max-pxcor + 1) file-write (max-pycor + 1) file-write maxGroupSize file-write equilibrium file-write groupExtinctionProb file-write localReplacement file-write innovationType file-write errorProb file-write refreshRate file-write Nm file-write intergroupTransmissionRadius file-write intergroupTransmissionType file-write rewireProbability
file-print " "
file-write "Time"
file-print " "
file-write ticks
file-print " "
file-print " "

file-print "Modal variants"
;;collect the mode of each group
 let x 0           ;;start at the left
 let y max-pycor   ;;start at the top

 while [y >= 0]
 [
 while [x <= max-pxcor]
 [
 file-write [modeCulturalVariant] of patch x y
 set x (x + 1)
 ]
 file-print " "
 set x 0
 set y (y - 1)
 ]
file-print " "


file-print "Mean distance to all other groups"
;;collect the mean distance to all other groups for each group
 set x 0
 set y max-pycor

 while [y >= 0]
 [
 while [x <= max-pxcor]
 [
 file-write [differenceToAllOtherGroups] of patch x y
 set x (x + 1)
 ]
 file-print " "
 set x 0
 set y (y - 1)
 ]
file-print " "


file-print "H"
;;collect the H for each group
 set x 0
 set y max-pycor

 while [y >= 0]
 [
 while [x <= max-pxcor]
 [
 file-write [H] of patch x y
 set x (x + 1)
 ]
 file-print " "
 set x 0
 set y (y - 1)
 ]
file-print " "


file-print "Shannon Index"
;;collect shannon index for each group
 set x 0
 set y max-pycor

 while [y >= 0]
 [
 while [x <= max-pxcor]
 [
 file-write [ShannonsIndexGroup] of patch x y
 set x (x + 1)
 ]
 file-print " "
 set x 0
 set y (y - 1)
 ]
file-print " "
file-close
end
@#$#@#$#@
GRAPHICS-WINDOW
291
103
662
475
-1
-1
36.3
1
10
1
1
1
0
1
1
1
0
9
0
9
1
1
1
ticks
30.0

BUTTON
7
44
73
77
setup
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
74
44
137
77
go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
139
44
202
77
step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
7
191
172
224
groupExtinctionProb
groupExtinctionProb
0
0.2
0.001
0.001
1
NIL
HORIZONTAL

SLIDER
7
157
129
190
maxGroupSize
maxGroupSize
0
100
25.0
5
1
NIL
HORIZONTAL

SLIDER
115
252
223
285
errorProb
errorProb
0
0.1
0.001
0.001
1
NIL
HORIZONTAL

PLOT
956
11
1227
193
Difference from Ancestral State (Histogram)
Difference from Ancestral
# of Groups
0.0
40.0
0.0
100.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

PLOT
664
10
949
192
Difference from Ancestral State
Time
Difference
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"mean" 1.0 0 -13345367 true "" ""
"sd" 1.0 0 -2674135 true "" ""
"var" 1.0 0 -10899396 true "" ""
"CV" 1.0 0 -11221820 true "" ""

SWITCH
131
157
274
190
localReplacement
localReplacement
0
1
-1000

SLIDER
7
10
116
43
seed
seed
0
10000
9.0
1
1
NIL
HORIZONTAL

PLOT
664
198
949
370
Difference from All Other Groups
Time
Difference from All Other Groups
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"mean" 1.0 0 -13345367 true "" ""
"sd" 1.0 0 -2674135 true "" ""
"var" 1.0 0 -10899396 true "" ""
"CV" 1.0 0 -11221820 true "" ""

PLOT
955
198
1226
370
Difference from All Other Groups (Histogram)
Mean Difference of Modes
# of Groups
0.0
20.0
0.0
100.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

CHOOSER
6
252
114
297
innovationType
innovationType
"forward-only" "bidirectional"
1

SLIDER
227
10
343
43
refreshRate
refreshRate
1
1000
1000.0
100
1
NIL
HORIZONTAL

SWITCH
117
10
226
43
collectData
collectData
1
1
-1000

PLOT
665
375
951
561
Population Cultural Diversity
Time
Diveristy
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"Total" 1.0 0 -16777216 true "" ""
"Within" 1.0 0 -2674135 true "" ""
"Between" 1.0 0 -13345367 true "" ""

PLOT
955
375
1226
561
Group Cultural Diversity
Time
H
0.0
10.0
0.0
1.0
true
true
"" ""
PENS
"mean" 1.0 0 -13345367 true "" ""
"sd" 1.0 0 -2674135 true "" ""
"var" 1.0 0 -10899396 true "" ""
"CV" 1.0 0 -11221820 true "" ""

MONITOR
555
57
660
102
Population Size
count turtles
0
1
11

TEXTBOX
317
482
645
538
When equilibrium = \"Off\", patch color signifies the following:\nDarker shades = mode is closer to ancestral state\nLighter shades = mode is further from ancestral state
11
0.0
1

SLIDER
177
330
269
363
Nm
Nm
0
maxGroupSize
1.0
1
1
NIL
HORIZONTAL

SLIDER
6
380
203
413
intergroupTransmissionRadius
intergroupTransmissionRadius
0
5
1.0
1
1
NIL
HORIZONTAL

CHOOSER
5
330
176
375
intergroupTransmissionType
intergroupTransmissionType
"spatial" "stochastic network" "global" "small world"
0

SLIDER
6
417
146
450
rewireProbability
rewireProbability
0
1
0.2
0.01
1
NIL
HORIZONTAL

MONITOR
591
10
660
55
Richness
totalDiversity
0
1
11

SWITCH
8
90
119
123
equilibrium
equilibrium
1
1
-1000

MONITOR
477
10
589
55
NIL
totalCopyErrors
0
1
11

MONITOR
390
57
554
102
NIL
totalIntergroupTransEvents
0
1
11

TEXTBOX
8
139
217
157
Local extinction and recolonization
11
0.0
1

TEXTBOX
8
234
158
252
Copying error
11
0.0
1

TEXTBOX
5
312
188
330
Intergroup cultural transmission
11
0.0
1

TEXTBOX
204
383
290
411
(used only with spatial network)
11
0.0
1

TEXTBOX
149
421
264
449
(used only with small world network)
11
0.0
1

@#$#@#$#@
# WHAT IS IT?

premo_2012.nlogo
(Updated in 2018 to run in NetLogo 6.0.2)

This model was used to collect the data presented in Premo (2012).  Note that the original version of this model was run on NetLogo 4.1.1.  That original code, which is no longer compatible with later versions of NetLogo, is still available upon request.  This version was updated in 2018 to run on NetLogo 6.0.2.  Because I had to change some of the code for this to run in the more recent version of NetLogo, the results of the BehaviorSpace experiments will not be exactly the same as those presented in the 2012 paper.  However, the differences are no more than would be expected with a different set of random number seeds--that is, the same major points still hold true qualitatively.  Note that the present version of the code may need to be modified further to run on subsequent versions of NetLogo.  If that is the case, please contact me, and I'd be happy to update it again.


# HOW TO USE IT

1. Use sliders and choosers on the Interface to set the parameters to your liking. If you would like to collect data in text files, switch collectData to "on".
2. Hit the "setup" button.
3. Hit the "go" button to run the model for up to 50,000 ticks, and hit "go" a second time to pause. If you hit "go" after 50,000 ticks, it will run until you hit "go" again to pause it.
4. To step through a simulation one time step at a time, hit the "step" button.


Replication: The same experiments I used in the 2012 paper are available in BehaviorSpace.  Note that the results won't be _identical_ to those presented in the original publication because I had to change some of the code syntax to make it compatible with NetLogo 6.0.2.  Nevertheless, the results are qualitatively the same.


# FULL MODEL DESCRIPTION (ODD PROTOCOL)

This model description follows the guidelines of the ODD protocol (Grimm et al. 2010). The model runs on NetLogo 6.0.2. The code may need to be modified to run on later versions of NetLogo.

## Purpose

This model is designed to address the following research question: How does the amount and topology of intergroup cultural transmission modulate the effect of local group extinction on selectively neutral cultural diversity in a geographically structured population?  The experimental design varies group extinction rate, the amount of intergroup cultural transmission, and the topology of intergroup cultural transmission while measuring the effects of local group extinction on long-term cultural change and regional cultural differentiation in a constant-size, spatially structured population.  The results show that for most of the intergroup social network topologies tested here, increasing the amount of intergroup cultural transmission (similar to increasing gene flow in a genetic model) erases the negative effect of local group extinction on selectively neutral cultural diversity. The stochastic (i.e., preference attachment) network seems to stand out as an exception.

## Agent classes, variables, & temporal and spatial scales
The model employs two classes of agents: individuals (turtles) and groups (patches).  Individuals serve as the vehicles of cultural transmission.  Individuals have only 2 variables: one representing whether they are members of the experienced or naive generation (age) and one representing a selectively neutral cultural variant (culturalVariant).  The age variable can be assigned one of two possible values: 0 (born during the current time step) or 1 (born during the previous time step).  culturalVariant can take the value of any integer, positive or negative.  Individuals have only two methods: die and cultTransmission.  An individual dies if its group goes extinct or if its age=1 after the completion of unbiased cultural transmission within groups.  The method, cultTransmission, defines the way in which each naive individual acquires its culturalVariant from a randomly chosen member of the experienced generation (i.e., an individual with age=1) within its group.

In the context of this model, groups are best described as spatially explicit containers of individuals.  Groups provide the population with spatial structure. Each group is characterized by a unique Cartesian coordinate as well as linkedPatches,
modeCulturalVariant, H, ShannonsIndexGroup, differenceFromAncestral, differenceToAllOtherGroups, withinGroupDiversity, fissions, and extinctions.  Groups have the following methods: intergroupTransmission, colonize, calculateWithinGroupPortionOfHt, calculateDifferenceFromAncestral, and calculateDifferenceToAllOtherGroups.

The model proceeds in simulated time steps (time steps are called "ticks" in Netlogo).  Each time step represents a single non-overlapping generation.  That is to say, by the end of each time step, the entire 'parental' or experienced generation--those individuals alive at the beginning of the time step--has been replaced by a new generation of 'offspring' or naive individuals. Note the non-overlapping generations aren't meant to represent actual human generations, but rather the time needed for 2 things to happen: 1) a cultural trait is passed from experienced practitioners to naive students within groups and 2) Nm individuals in each group replace the culturalVariant they adopted through unbiased cultural transmission with the culturalVariant they acquire through horizontal transmission between groups.

The model is spatially explicit.  The spatial environment is a grid of regularly spaced and shaped square cells.  The edges of the grid are wrapped into a torus to avoid edge effects.  Although cell size does not equate to an empirical value, one can think of it as the area needed to support the subsistence activities of a single social group within an idealized homogeneous landscape.  Thus, each cell represents a single group, which can contain up to maxGroupSize individuals.

## Process Overview and Scheduling
Each time step is composed of 6 distinct processes, which occur in the following order: individuals get older, groups are exposed to risk of local extinction, the naive generation is created, cultural variants are transmitted from the experienced generation to the naive generation within groups, the experienced generation is culled, and intergroup cultural transmission occurs among members of the naive generation.

First, at the very beginning of each time step, each individual (i.e., turtle) changes the value of its age variable from 0 to 1 to reflect the fact that it is now a member of the "experienced" generation and can serve as a teacher during oblique cultural transmission later in the time step.

Second, each group is exposed to local extinction and recolonization.  If a real number chosen randomly from a uniform distribution bound by 0 and 1 is less than groupExtinctionProb, then the individuals who are members of that group are removed from the simulation immediately.  Following a local extinction event, the cell is empty.  Each empty cell is repopulated with members of the experienced generation taken from a randomly chosen neighboring cell when localReplacement = true or a randomly chosen cell when localReplacement = false. Empty cells begin to be repopulated only after all groups have been exposed to a chance of local extinction.

Third, the naive generation is created; 25 new individuals are created in each group.  Members of the naive generation are distinguished from those in the experienced generation by age (age=0 for naive generation, age=1 for experienced generation).

Fourth, each member of the naive generation undergoes unbiased cultural transmission within its group.  Unbiased cultural transmission involves (1) learning from a teacher who is drawn randomly (with replacement) from among the members of the experienced generation within the same group and (2) being exposed to the probability (errorProb) of making an error when copying the teacher's culturalVariant.  Copy errors always result in the naive individual adopting (with equal probability) either x-1 or x+1 as its culturalVariant, where x represents the value of the teacher's culturalVariant.  This is the bidirectional version of the single stepwise mutation model.  See Submodels for more details on cultural transmission and copy errors.

Fifth, all remaining members of the experienced generation (i.e., age=1) are culled from the simulation.  The population size (2500) is the same as it was at the beginning of the time step, but now all of the individuals are members of the naive generation (i.e., age=0).

Sixth, Nm individuals in each group undergo horizontal intergroup cultural transmission. 'N' refers to the maximum number of individuals in the group and 'm' refers to the probability of learning via intergroup cultural transmission. Intergroup cultural transmission involves each of the Nm individuals in each group learning from a teacher who is drawn randomly (with replacement) from among the members of the naive generation in a group that is "linked" to the learner's group.  The topology of the intergroup social network can take one of four forms: global (essentially the teacher is chosen randomly from any group that is not the learner's), spatial (the teacher is chosen randomly from any of the groups that are located within intergroupTransmissionRadius of the learner's group), stochastic network (the teacher is chosen randomly from any group that is linked the learner's group in a preference attachment network), or small world (the teacher is chosen randomly from any group that is linked the learner's group in a small world network with rewireProbability=0.2). Note that, unlike the case of unbiased cultural transmission within groups, intergroup cultural transmission does not include copying error.  Please see intergroupTransmission in Submodels for more details.

## Design Concepts

Evolution.  In the context of this model, evolution refers to change in cultural variant frequencies in a population through time.  Because the cultural variants are selectively neutral (i.e., non-adaptive) in this model, the results speak only to neutral cultural evolution.

Stochasticity. Stochasticity is incorporated into many aspects of this nondeterministic model: copy errors and localized group extinctions are represented probabilistically, agent order is randomized before every process, teachers are chosen randomly from among the set of candidates, and the groups tabbed to fission in order to repopulate an empty group are chosen randomly from among the empty group's Moore neighbors. The small world and preference attachment social networks also include stochastic elements.

## Initialization

At the start of each simulation, all groups (100) possess the maximum number of individuals (25), for a total of 2500 individuals. When equilibrium=False, every one of these individuals displays culturalVariant=0 at the start of the simulation run.  Under this condition, the initial population is perfectly homogeneous in terms of cultural diversity; there is no variation within or between groups.  When equilibrium=True, every individual displays a culturalVariant value chosen at random from a uniform distribution of integers between 0 and 10^8.  Under this condition, the initial population is heterogeneous in terms of cultural diversity and there is maximum variation within and between groups at the start of the run.  Table 1 provides the values used to initialize the simulations reported in the paper.

Table 1. Parameter values used to initialize the simulations reported in Premo (2012).
________________________________________________________________
Standard parameters
   maxGroupSize..........................25
   errorProb.................................0.001
   innovationType..........................bidirectional
   intergroupTransmissionRadius.....1
   rewireProbability........................0.20
   localReplacement......................True
   equilibrium................................True, False

Experimental parameters
   groupExtinctionProb...................0, 0.001, 0.01, 0.1
   Nm..........................................0, 1, 2, 3, 4, 5 
   intergroupTransmissionType........spatial, global, stochastic, small world
   seed........................................1-20

________________________________________________________________


## Input Data

The model does not require any input data or additional files to initialize a simulation run.

## Data Collection

Data are collected from each simulation after 50,000 time steps.  I collect variant richness (the total number of unique cultural variants displayed by the entire population), total diversity (H_T), within-group diversity (H_S), F_ST (the proportion of total diversity explained by between-group diversity), dbar_G, and d_Amax.  Please see Premo (2012) and the source code for the formal definitions of these measures of cultural variation, group differentiation, and rates of cumulative change.

## Submodels

### Creating a scale-free stochastic social network (establishLink)
This is a patches method used to create a stochastic (i.e., preference attachment) network, as described in Bentley and Shennan (2003).  The general idea here is that, as the social network is created, the likelihood that a link is created between a newly added unlinked node any extant node is a function of the number of links at the extant nodes.  This is a rich-get-richer type of network in which nodes with more links are more likely to attract additional links than nodes with fewer links.

Each patch goes through these steps after it adds 25 turtles at the start of a simulation run.
1. All other patches that contain turtles are added to a local agentset called possibleLinks.
2. Each member of possibleLinks adds its plabel to tempList X times, where X represents the number of links it currently has.  For example, a patch with plabel = 3 that is linked to 5 other patches would add the number '3' to tempList 5 times while a patch with plabel = 4 that is linked to 2 other patches would add the number '4' to tempList 2 times.
3. After all of the members of possibleLinks have populated tempList with their plabels in quantities that correspond to the number of links they currently have to other patches, then one item is randomly chosen from tempList.  This item represents the plabel of the patch to which ego will be connected via a link in the social network.  The link is created by setting ego's list of linkedPatches to include the patch that matches the plabel drawn from tempList and by asking the newly linked patch to add ego to its list of linkedPatches.

### Recolonizing empty cells (colonize)  
As described above, when a group undergoes a localized extinction event, all of the individuals within it are removed from the simulation.  To repopulate the newly empty cell with some members of the experienced generation who will serve as possible teachers for the group's naive generation, a group from the empty cell's Moore neighborhood is randomly chosen to fission.  Aside from being adjacent to the empty cell, the only other requirement for serving as a source of individuals is that the group must have at least 2 members.  Once chosen, the group sends half of its individuals (chosen randomly from the agents in the cell) to 'recolonize,' or repopulate, the empty cell. These 'colonizers' will serve as the experienced generation in the cell to which they have moved.

### Unbiased cultural transmission within groups (cultTransmission)   
Intergenerational cultural transmission occurs within spatially defined groups.  Every unbiased cultural transmission event involves a naive 'offspring' who has yet to adopt a culturalVariant and an experienced teacher from the 'parental' generation.  The naive individual copies the culturalVariant expressed by its teacher.  Each naive individual randomly chooses its teacher (with replacement) from the set of individuals who are members of the 'parental' generation in the naive individual's spatially defined group.

Unbiased cultural transmission is imperfect.  A naive individual makes a mistake when copying its teacher's variant with probability errorProb.  Each copying error results in the naive individual adopting a cultural variant that is one integer greater or lower (with equal probability) than the value it was trying to copy from its teacher.  In the parlance of population genetics, the effect of copy errors on cultural variants follows a bidirectional single stepwise mutation model.  The model also allows for one to investigate a unidirectional, or forward-only, single stepwise mutation model.

### Intergroup cultural transmission (intergroupTransmission)
Intergroup cultural transmission occurs between members of different groups.  Put simply, each instance of intergroup cultural transmission is marked by a member of one group adopting the culturalVariant of a member of a different group.  This occurs Nm times for each group during each time step.  Most of the code in this submodel is used to delineate from which groups one's teacher can be drawn, as this is based on the topology of the social network that connects groups.
 
When intergroupTransmissionType=spatial, each "learner" in a group chooses its teacher randomly (and with replacement) from among the other groups (patches) that are located within the distance given by intergroupTransmissionRadius. Note that the subset of groups from which the learner may choose a teacher (linkedPatches) doesn't include the learner's own group.  When intergroupTransmissionRadius=1, as it does in Premo (2012), this equates to the learner choosing a teacher from one of its von Neumann neighbors.  This is referred to as the "local" social network topology in Premo (2012).

When intergroupTransmissionType=global, each "learner" in a group chooses its teacher randomly (and with replacement) from among the members of any other group. The only group from which the learner may not choose a teacher is its own group.  This topology represents the idealized case in which all groups are equally connected to all others and the spatial distance between groups plays no role. 

When intergroupTransmissionType=stochastic network, each "learner" in a group chooses its teacher randomly (and with replacement) from among the groups (patches) that are members of its group's linkedPatches list.  Note that each group's linkedPatches list does not include itself--i.e., a group cannot be "linked" to itself in an intergroup social network.

When intergroupTransmissionType=small world, each "learner" in a group chooses its teacher randomly (and with replacement) from among the groups (patches) that are members of its group's linkedPatches list.  Note that each group's linkedPatches list does not include itself--i.e., a group cannot be "linked" to itself in an intergroup social network.  In the case of the small world network, each group's linkedPatches list is defined only after all groups have been initialized at the start of the simulation. The small world social network topology makes use of the global variable, rewireProbability.  rewireProbability represents the proportion of patches that will reset, or “rewire,” one of its links such that it replaces a link to a von Neumann neighbor with a link to a patch that is not a von Neumann neighbor.

In this model, intergroup cultural transmission does not include copying error.



## Noteworthy model assumptions
This model does not allow for horizontal cultural transmission within groups. Naive individuals can only learn from members of the previous generation--never from members of their own generation--within their own group. It is unclear how including horizontal transmission within groups would affect the results, and this might be worth investigating further. Although intergroup cultural transmission is a form of horizontal cultural transmission, this occurs between members of different groups.  Intergroup cultural transmission does not include copying error.



# THINGS TO TRY

Switch localReplacement to OFF to investigate the effects of local extinction and intergroup network topology under global recolonization.


# REFERENCES

Bentley, R. A. and S. J. Shennan (2003) Cultural transmission and stochastic network growth. _American Antiquity_ 68:459-485.

Grimm, V., et al. (2010) The ODD protocol: A review and first update. _Ecological Modelling_ 221:2760-2768.

Premo, L. S. (2012) Local extinctions, connectedness, and cultural evolution in a structured population. _Advances in Complex Systems_ 15:1150002. DOI: 10.1142/S0219525911003268.


# CONTACT INFORMATION

This model was programmed by Luke Premo (Department of Anthropology, Washington State University and the Department of Human Evolution, Max Planck Institute for Evolutionary Anthropology). One of the best things about computer simulation programs is that they can often be improved with the help of other researchers. The more sets of eyes that pass over the code, the better. So, please, have a look at the commented source code under the "info" tab. Any comments, questions, or corrections are always welcome. Feel free to contact me at:

Luke Premo
Department of Anthropology
Washington State University
Pullman, WA 99164-4910
luke.premo at wsu dot edu
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270
@#$#@#$#@
NetLogo 6.0.2
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="exp_errorProb001_local" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="50001"/>
    <metric>ticks</metric>
    <metric>count turtles</metric>
    <metric>totalDiversity</metric>
    <metric>ShannonsIndex</metric>
    <metric>H_t</metric>
    <metric>H_s</metric>
    <metric>t_f</metric>
    <metric>meanDiffFromAncestral</metric>
    <metric>CVDiffFromAncestral</metric>
    <metric>maxDiffFromAncestral</metric>
    <metric>totalCultTransEvents</metric>
    <metric>totalGroupExtinctions</metric>
    <metric>totalGroupFissions</metric>
    <metric>meanNumberGroupFissions</metric>
    <metric>varNumberGroupFissions</metric>
    <metric>meanHofGroups</metric>
    <metric>varHofGroups</metric>
    <metric>meanDiffOtherGroups</metric>
    <metric>CVDiffOtherGroups</metric>
    <metric>meanSIofGroups</metric>
    <metric>varSIofGroups</metric>
    <enumeratedValueSet variable="maxGroupSize">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="refreshRate">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collectData">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="innovationType">
      <value value="&quot;bidirectional&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="localReplacement">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionType">
      <value value="&quot;spatial&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="errorProb">
      <value value="0.001"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionRadius">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="equilibrium">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groupExtinctionProb">
      <value value="0"/>
      <value value="0.001"/>
      <value value="0.01"/>
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Nm">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
      <value value="4"/>
      <value value="5"/>
    </enumeratedValueSet>
    <steppedValueSet variable="seed" first="1" step="1" last="20"/>
  </experiment>
  <experiment name="exp_errorProb001_global" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="50001"/>
    <metric>ticks</metric>
    <metric>count turtles</metric>
    <metric>totalDiversity</metric>
    <metric>ShannonsIndex</metric>
    <metric>H_t</metric>
    <metric>H_s</metric>
    <metric>t_f</metric>
    <metric>meanDiffFromAncestral</metric>
    <metric>CVDiffFromAncestral</metric>
    <metric>maxDiffFromAncestral</metric>
    <metric>totalCultTransEvents</metric>
    <metric>totalGroupExtinctions</metric>
    <metric>totalGroupFissions</metric>
    <metric>meanNumberGroupFissions</metric>
    <metric>varNumberGroupFissions</metric>
    <metric>meanHofGroups</metric>
    <metric>varHofGroups</metric>
    <metric>meanDiffOtherGroups</metric>
    <metric>CVDiffOtherGroups</metric>
    <metric>meanSIofGroups</metric>
    <metric>varSIofGroups</metric>
    <enumeratedValueSet variable="maxGroupSize">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="refreshRate">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collectData">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="innovationType">
      <value value="&quot;bidirectional&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="localReplacement">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionType">
      <value value="&quot;global&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="errorProb">
      <value value="0.001"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionRadius">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="equilibrium">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groupExtinctionProb">
      <value value="0"/>
      <value value="0.001"/>
      <value value="0.01"/>
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Nm">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
      <value value="4"/>
      <value value="5"/>
    </enumeratedValueSet>
    <steppedValueSet variable="seed" first="1" step="1" last="20"/>
  </experiment>
  <experiment name="exp_errorProb001_stochnetwork" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="50001"/>
    <metric>ticks</metric>
    <metric>count turtles</metric>
    <metric>totalDiversity</metric>
    <metric>ShannonsIndex</metric>
    <metric>H_t</metric>
    <metric>H_s</metric>
    <metric>t_f</metric>
    <metric>meanDiffFromAncestral</metric>
    <metric>CVDiffFromAncestral</metric>
    <metric>maxDiffFromAncestral</metric>
    <metric>totalCultTransEvents</metric>
    <metric>totalGroupExtinctions</metric>
    <metric>totalGroupFissions</metric>
    <metric>meanNumberGroupFissions</metric>
    <metric>varNumberGroupFissions</metric>
    <metric>meanHofGroups</metric>
    <metric>varHofGroups</metric>
    <metric>meanDiffOtherGroups</metric>
    <metric>CVDiffOtherGroups</metric>
    <metric>meanSIofGroups</metric>
    <metric>varSIofGroups</metric>
    <enumeratedValueSet variable="maxGroupSize">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="refreshRate">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collectData">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="innovationType">
      <value value="&quot;bidirectional&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="localReplacement">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionType">
      <value value="&quot;stochastic network&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="errorProb">
      <value value="0.001"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionRadius">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="equilibrium">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groupExtinctionProb">
      <value value="0"/>
      <value value="0.001"/>
      <value value="0.01"/>
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Nm">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
      <value value="4"/>
      <value value="5"/>
    </enumeratedValueSet>
    <steppedValueSet variable="seed" first="1" step="1" last="20"/>
  </experiment>
  <experiment name="exp_errorProb001_smallworld" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="50001"/>
    <metric>ticks</metric>
    <metric>count turtles</metric>
    <metric>totalDiversity</metric>
    <metric>ShannonsIndex</metric>
    <metric>H_t</metric>
    <metric>H_s</metric>
    <metric>t_f</metric>
    <metric>meanDiffFromAncestral</metric>
    <metric>CVDiffFromAncestral</metric>
    <metric>maxDiffFromAncestral</metric>
    <metric>totalCultTransEvents</metric>
    <metric>totalGroupExtinctions</metric>
    <metric>totalGroupFissions</metric>
    <metric>meanNumberGroupFissions</metric>
    <metric>varNumberGroupFissions</metric>
    <metric>meanHofGroups</metric>
    <metric>varHofGroups</metric>
    <metric>meanDiffOtherGroups</metric>
    <metric>CVDiffOtherGroups</metric>
    <metric>meanSIofGroups</metric>
    <metric>varSIofGroups</metric>
    <enumeratedValueSet variable="maxGroupSize">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="refreshRate">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collectData">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="innovationType">
      <value value="&quot;bidirectional&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="localReplacement">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionType">
      <value value="&quot;small world&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="errorProb">
      <value value="0.001"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="rewireProbability">
      <value value="0.2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="equilibrium">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groupExtinctionProb">
      <value value="0"/>
      <value value="0.001"/>
      <value value="0.01"/>
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Nm">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
      <value value="4"/>
      <value value="5"/>
    </enumeratedValueSet>
    <steppedValueSet variable="seed" first="1" step="1" last="20"/>
  </experiment>
  <experiment name="exp_errorProb001_Nm0" repetitions="1" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="50001"/>
    <metric>ticks</metric>
    <metric>count turtles</metric>
    <metric>totalDiversity</metric>
    <metric>ShannonsIndex</metric>
    <metric>H_t</metric>
    <metric>H_s</metric>
    <metric>t_f</metric>
    <metric>meanDiffFromAncestral</metric>
    <metric>CVDiffFromAncestral</metric>
    <metric>maxDiffFromAncestral</metric>
    <metric>totalCultTransEvents</metric>
    <metric>totalGroupExtinctions</metric>
    <metric>totalGroupFissions</metric>
    <metric>meanNumberGroupFissions</metric>
    <metric>varNumberGroupFissions</metric>
    <metric>meanHofGroups</metric>
    <metric>varHofGroups</metric>
    <metric>meanDiffOtherGroups</metric>
    <metric>CVDiffOtherGroups</metric>
    <metric>meanSIofGroups</metric>
    <metric>varSIofGroups</metric>
    <enumeratedValueSet variable="maxGroupSize">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="refreshRate">
      <value value="10000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="collectData">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="innovationType">
      <value value="&quot;bidirectional&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="localReplacement">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionType">
      <value value="&quot;global&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="errorProb">
      <value value="0.001"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="intergroupTransmissionRadius">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="equilibrium">
      <value value="true"/>
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="groupExtinctionProb">
      <value value="0"/>
      <value value="0.001"/>
      <value value="0.01"/>
      <value value="0.1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Nm">
      <value value="0"/>
    </enumeratedValueSet>
    <steppedValueSet variable="seed" first="1" step="1" last="20"/>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180
@#$#@#$#@
0
@#$#@#$#@
